# -*- coding: utf-8 -*-

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import logging

from adles.utils import time_execution


class Interface:
    """ Generic interface used to uniformly interact with platform-specific interfaces. """

    def __init__(self, spec):
        """ :param spec: Full specification """
        from sys import exit
        self.metadata = spec["metadata"]

        # Load infrastructure information from the configuration file
        from adles.parser import parse_file
        infrastructure = parse_file(self.metadata["infrastructure-config-file"])
        if "server-hostname" not in infrastructure:
            infrastructure["server-hostname"] = str(input("Enter hostname for infrastructure: "))
        if "server-port" not in infrastructure:
            infrastructure["server-port"] = int(input("Enter port for infrastructure server: "))
        if "login-file" not in infrastructure:
            infrastructure["login-file"] = str(input("Enter filename for infrastructure logins: "))

        # Load infrastructure login information from the file specified in infrastructure config
        from adles.utils import read_json
        logging.debug("Loading login information from file %s", infrastructure["login-file"])
        logins = read_json(infrastructure["login-file"])
        if not logins:
            exit(1)

        # Select the Interface to use based on the specified infrastructure platform
        if infrastructure["platform"] == "vmware vsphere":
            from .vsphere_interface import VsphereInterface
            self.interface = VsphereInterface(infrastructure, logins, spec)
        else:
            logging.error("Invalid platform: %s", infrastructure["platform"])
            exit(1)

    @time_execution
    def create_masters(self):
        """ Master creation phase """
        logging.info("Creating Master instances for %s", self.metadata["name"])
        self.interface.create_masters()

    @time_execution
    def deploy_environment(self):
        """ Environment deployment phase """
        logging.info("Deploying environment for %s", self.metadata["name"])
        self.interface.deploy_environment()

    @time_execution
    def cleanup_masters(self, network_cleanup=False):
        """
        Cleans up master instances
        :param network_cleanup: If networks should be cleaned up [default: False]
        """
        logging.info("Cleaning up Master instances for %s", self.metadata["name"])
        self.interface.cleanup_masters(network_cleanup=network_cleanup)

    @time_execution
    def cleanup_environment(self, network_cleanup=False):
        """
        Cleans up a deployed environment
        :param network_cleanup: If networks should be cleaned up [default: False]
        """
        logging.info("Cleaning up environment for %s", self.metadata["name"])
        self.interface.cleanup_masters(network_cleanup=network_cleanup)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""vsphere_testing. Entrypoint used to test the vSphere class. Has examples of API usage.

Usage:
    vsphere_testing.py [-v] [options]
    vsphere_testing.py --version
    vsphere_testing.py (-h | --help)

Options:
    -h, --help          Shows this help
    --version           Prints current version
    --no-color          Do not color terminal output
    -v, --verbose       Emit debugging logs to terminal
    -f, --file FILE     Name of JSON file with server connection information

"""

import logging

from docopt import docopt
# from pyVmomi import vim

from adles.vsphere.vsphere_class import Vsphere
from adles.vsphere import vm_utils
from adles.utils import script_setup


args = docopt(__doc__, version=Vsphere.__version__, help=True)
server = script_setup('vsphere_testing.log', args, (__file__, Vsphere.__version__))

folder = server.get_folder("monkeys")


# bios_uuid = "42053029-1098-6ef1-da79-5e2c4103f600"
# instance_uuid = "50056cfe-aaa0-c28d-a702-d72e68e50f3a"
# vm = server.find_by_uuid(uuid=instance_uuid)
# vm = server.find_by_ds_path(path="[Datastore] (MASTER) apache/(MASTER) apache.vmx")
# logging.info("%s", vm_utils.get_vm_info(vm, uuids=True))

# pool = server.get_item(vim.ResourcePool)
# vm_spec = vm_utils.gen_vm_spec(name="test_vm", datastore_name="Datastore",
#                                annotation="testing 1...")
# vm_utils.create_vm(folder, vm_spec, pool)
# attach_iso(vm, "ISO-Images/vyos-1.1.7-amd64.iso", server.get_datastore("Datastore"))

vm = server.get_vm("real-vm")
logging.info(vm_utils.get_vm_info(vm, detailed=True, uuids=True, snapshot=True, vnics=True))

# vms = server.get_all_vms()
# for vm in vms:
#     logging.info(vm_utils.get_vm_info(vm))

# net = server.get_network("ARP-LAN")
# logging.info(str(net))

# host = server.get_host()
# logging.info(host)

# logging.info(str(server))
# logging.info(repr(server))

# from adles.vsphere.folder_utils import traverse_path
# folder = server.get_folder("script_testing")
# vm = traverse_path(folder, "/Templates/Routers/VyOS 1.1.7 (64-bit)")
# logging.info(get_vm_info(vm))

# datastore = server.get_datastore("Datastore")
# logging.info(get_datastore_info(datastore))

# portgroup = server.get_item(vim.Network, "test_network")
# logging.info("Portgroup: %s", str(portgroup))
# vm_utils.add_nic(vm, portgroup, "test_summary")
# vm_utils.edit_nic(vm, 2, summary="lol")
# vm_utils.delete_nic(vm, 1)

# create_portgroup("test_portgroup", server.get_host(), "test_vswitch")
# delete_portgroup("test_portgroup", server.get_host())



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# http://multivax.com/last_question.html

# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""ADLES: Automated Deployment of Lab Environments System.
Uses YAML specifications to manage and create virtual environments.

Usage:
    adles.py [options] -c SPEC
    adles.py [options] (-m | -d) [-p] -s SPEC
    adles.py [options] (--cleanup-masters | --cleanup-enviro) [--network-cleanup] -s SPEC
    adles.py --version
    adles.py (-h | --help)

Options:
    --no-color                  Do not color terminal output
    -v, --verbose               Emit debugging logs to terminal
    -c, --check-syntax SPEC     Validates syntax of an exercise specification
    -s, --spec SPEC             YAML file with the exercise environment specification
    -p, --package               Build environment from package specification
    -m, --create-masters        Master creation phase of specification
    -d, --deploy                Environment deployment phase of specification
    --cleanup-masters           Cleanup masters created by a specification
    --cleanup-enviro            Cleanup environment created by a specification
    --network-cleanup           Cleanup networks created during either phase
    -i, --infra FILE            Specifies the infrastructure configuration file to use
    -h, --help                  Shows this help
    --version                   Prints current version

Examples:
    adles.py --check-syntax examples/tutorial.yaml
    adles.py --verbose --create-masters --spec examples/experiment.yaml
    adles.py -v -d -s examples/competition.yaml
    adles.py --cleanup-masters --network-cleanup -s examples/competition.yaml

"""

import logging

from docopt import docopt
from pyVmomi import vim

from adles.interfaces import Interface
from adles.parser import check_syntax
from adles.utils import setup_logging
from adles import __version__


def main():
    args = docopt(__doc__, version=__version__, help=True)
    colors = (False if args["--no-color"] else True)
    setup_logging(filename='adles.log', colors=colors, console_verbose=args["--verbose"])

    if args["--spec"]:
        if args["--package"]:
            # TODO: implement basic extraction of environment spec from package spec
            logging.error("Package specifications are not implemented yet")
            exit(1)

        spec = check_syntax(args["--spec"])
        if spec is None:
            logging.error("Syntax check failed")
            exit(1)
        if args["--infra"]:
            from os.path import exists
            infra_file = args["--infra"]
            if not exists(infra_file):
                logging.error("Specified infrastructure config file '%s' could not be found,"
                              " falling back to exercise configuration", infra_file)
            else:
                logging.debug("Overriding infrastructure config file with '%s'", infra_file)
                spec["metadata"]["infrastructure-config-file"] = infra_file

        try:
            interface = Interface(spec)
            if args["--create-masters"]:
                interface.create_masters()
                logging.info("Finished creation of Masters for environment %s",
                             spec["metadata"]["name"])
            elif args["--deploy"]:
                interface.deploy_environment()
                logging.info("Finished deployment of environment %s", spec["metadata"]["name"])
            elif args["--cleanup-masters"]:
                interface.cleanup_masters(args["--network-cleanup"])
            elif args["--cleanup-enviro"]:
                interface.cleanup_environment(args["--network-cleanup"])
            else:
                logging.error("Invalid flags for --spec. Argument dump:\n%s", str(args))
        except vim.fault.NoPermission as e:
            logging.error("Permission error: \n%s", str(e))
            exit(1)
        except AttributeError:
            logging.error("Critical fault: unexpected None-type object. Exiting...")
            exit(1)
    elif args["--check-syntax"]:
        if check_syntax(args["--check-syntax"]) is None:
            logging.error("Syntax check failed")
    else:
        logging.error("Invalid arguments. Argument dump:\n%s", str(args))

if __name__ == '__main__':
    main()
